package JSV;

use 5.008005;
use strict;
use warnings;

our $VERSION = "0.05";

1;
__END__

=encoding utf-8

=head1 NAME

JSV - JSON Schema implementation of Perl

=head1 SYNOPSIS

    use JSV;

=head1 DESCRIPTION

See e.g. L<JSV::Validator>

=head1 LICENSE

Copyright (C) zigorou.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 AUTHOR

zigorou E<lt>zigorou@cpan.orgE<gt>

=cut

